package com.care.demo.controller;

import cn.hutool.json.JSONUtil;
import com.care.common.vo.PageResVO;
import com.care.demo.entity.RimsDictionaries;
import com.care.demo.service.SystemDictionariesService;
import com.care.common.util.PageResult;
import com.care.common.util.PageUtil;
import com.care.common.util.Result;
import com.care.common.util.ResultCode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * @author javadoc
 * @version 1.0
 * @description 系统设置-字典管理模块-前端控制器
 * @date
 */
@Slf4j
@RestController
@RequestMapping("/bms/system/dictionaries")
public class SystemDictionariesController {

    @Autowired
    SystemDictionariesService systemDictionariesService;


    /**
     * 字典：分页查询
     * 作用：从前端获取查询条件(可无条件)和分页数据，请求数据库获取数据
     *
     * @param rimsDictionaries 收集到的多条件字典查询对象
     * @param page             分页对象（收集分页参数）
     * @return 无搜索条件时返回的是有父子级结构的字典数据，有搜索条件时返回的时无父子结构的字典数据
     */
    @GetMapping("/getDictionariesByPage")
    public PageResult<List<RimsDictionaries>> getDictionariesByPage(RimsDictionaries rimsDictionaries, PageResVO page) {
        try {
            //声明字典多条件查询分页对象集合（数据包含在分页工具类中）
            PageUtil<RimsDictionaries> dictionariesByPage;
            //若字典名称搜索条件为空则为无条件分页查询（返回数据包含父子级结构）
            if(StringUtils.isBlank(rimsDictionaries.getDictionariesName())){
                dictionariesByPage =
                        systemDictionariesService.getDictionaries(page);
            } else {
                //获取字典多条件查询分页对象集合（数据包含在分页工具类中）
                dictionariesByPage =
                        systemDictionariesService.getDictionariesByPage(rimsDictionaries, page);
            }
            if (dictionariesByPage == null) {
                return PageResult.error( "数据错误，获取字典数据失败");
            } else {

                return  PageResult.success(
                        "查询成功",dictionariesByPage.getListT(),
                        dictionariesByPage.getCurrent(),
                        dictionariesByPage.getPageSize(),
                        dictionariesByPage.getTotal());
            }
        } catch (Exception e) {
           log.error("字典:分页查询出现异常",e);
            return PageResult.error("系统错误，获取字典数据失败");
        }
    }


    /**
     * 字典：字段查重（字典选项代码）
     * 作用：添加或修改时对必须唯一的字段进行查重
     *
     * @param rimsDictionaries 收集需要查重的字段参数
     * @return 返回0表示字段没有重复，非零表示有重复
     */
    @GetMapping("/getDictionaRepetitionCount")
    public Result<Object> getDictionaRepetitionCount(RimsDictionaries rimsDictionaries) {
        try {
            Integer repetitionCount =
                    systemDictionariesService.getDictionaRepetitionCount(rimsDictionaries);
            if (repetitionCount == 0) {
                return Result.success(
                        "字段查重通过");
            }else{
                return new Result<>(ResultCode.ERROR,
                        "字段查重不通过");
            }
        } catch (Exception e) {
           log.error("字典:字典查重出现异常",e);
            return Result.error("系统错误，查询失败");
        }
    }


    /**
     * 字典：添加记录
     * 作用：添加字典数据
     *
     * @param data 收集参数的json对象
     * @return 添加成功返回 "00000"
     */
    @PostMapping(value = "/addDictionaries",produces = "application/json; charset=utf-8")
    public Result<Object> addDictionaries(@RequestBody String data) {
        try {
            RimsDictionaries rimsDictionaries = JSONUtil.parse(data).toBean(RimsDictionaries.class);
            //参数校验
            Result<Object> result = checkDictionariesParameter(rimsDictionaries);
            if (result != null) {
                return result;
            }
            Integer indexFlag = systemDictionariesService.addDictionaries(rimsDictionaries);
            return indexFlag == 1 ?
                    Result.success("添加成功") :
                    new Result<>(ResultCode.ERROR,"数据错误，添加失败") ;
        } catch (Exception e) {
           log.error("字典:添加出现异常",e);
            return Result.error("系统错误，添加失败");
        }
    }


    /**
     * 字典：修改记录
     * 作用：修改字典数据
     *
     * @param data 收集参数的json对象
     * @return 添加成功返回 "00000"
     */
    @PutMapping(value = "/updateDictionaries",produces = "application/json; charset=utf-8")
        public Result<Object> updateDictionaries(@RequestBody String data) {
        try {
            RimsDictionaries rimsDictionaries = JSONUtil.parse(data).toBean(RimsDictionaries.class);
            //参数校验字典ID是否为空
            if(rimsDictionaries.getId() == null){
                return new Result<>(ResultCode.ERROR,"必要参数ID不能为空");
            }
            //参数校验
            Result<Object> result = checkDictionariesParameter(rimsDictionaries);
            if (result != null) {
                return result;
            }
            Integer indexFlag = systemDictionariesService.updateDictionaries(rimsDictionaries);
            return indexFlag == 1 ?
                    Result.success("修改成功") :
                    new Result<>(ResultCode.ERROR,"数据错误，修改失败") ;
        } catch (Exception e) {
           log.error("字典:修改出现异常",e);
            return Result.error("系统错误，修改字典失败");
        }
    }


    /**
     * 字典：删除记录
     * 作用：删除字典数据
     *
     * @param id 要删除字典的id
     * @return 删除成功返回 "00000"
     */
    @DeleteMapping(value = "/deleteDictionariesById/{id}",produces = "application/json; charset=utf-8")
    public Result<Object> deleteDictionariesById(@PathVariable Long id){
        try{
            Integer flag = systemDictionariesService.deleteDictionariesById(id);
            if (flag == 0){
                return Result.error("删除失败");
            }else{
                return Result.success( "删除成功") ;
            }
        }catch (Exception e){
           log.error("字典:删除出现异常",e);
            return Result.error("系统错误，删除失败");
        }
    }


    /**
     * 字典：启用/停用状态修改
     * 作用：启用或停用某个字典记录数据
     *
     * @param data 收集参数的json对象(里面包含，id，pid，enable 三个参数)
     * @return 删除成功返回 "00000"
     */
    @PutMapping(value = "/updateDictionariesEnableById",produces = "application/json; charset=utf-8")
    public Result<Object> updateDictionariesEnableById(@RequestBody String data){
        try{
            //接收到参数转化成字典对象
            RimsDictionaries rimsDictionaries = JSONUtil.parse(data).toBean(RimsDictionaries.class);
            //参数校验
            if(rimsDictionaries.getId() == null){
                return new Result<>(ResultCode.ERROR,"缺少必要参数ID");
            }
            if(rimsDictionaries.getPid() == null){
                return new Result<>(ResultCode.ERROR,"缺少必要参数PID");
            }
            if(rimsDictionaries.getEnable() == null){
                return new Result<>(ResultCode.ERROR,"必要参数更新状态不能为null");
            }
            Integer flag = systemDictionariesService.updateDictionariesEnableById(rimsDictionaries);
            if (flag == 0){
                return Result.error("修改失败");
            }else{
                return Result.success( "修改成功") ;
            }
        }catch (Exception e){
           log.error("字典:状态更新出现异常",e);
            return Result.error("系统错误，状态更新失败");
        }
    }




    /**
     * 字典：校验必传参数
     * 作用：添加和修改时确保关键字段非空,以及选项代码不为空时查重
     *
     * @param rimsDictionaries 要验证的参数对象
     * @return 返回校验结果
     */
    private Result<Object> checkDictionariesParameter(RimsDictionaries rimsDictionaries) {
        log.info("字典参数校验传参为 = " + rimsDictionaries);
        if (rimsDictionaries.getPid() == null) {
            return Result.error("参数PID不能为空");
        }
        if (StringUtils.isBlank(rimsDictionaries.getDictionariesName())) {
            return Result.error("字典名称不能为空");
        }
        if(StringUtils.isNotBlank(rimsDictionaries.getDicCode())
                && systemDictionariesService.getDictionaRepetitionCount(rimsDictionaries) != 0){
            return Result.error("选项代码重复");
        }
        return null;
    }


}
